@@ -4,8 +4,6 @@
 srcdir=`dirname $0`
 test -z "$srcdir" && srcdir=.
 
-PKG_NAME="evince"
-
 (test -f $srcdir/configure.ac) || {
     echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
     echo " top-level $PKG_NAME directory"
@@ -19,5 +17,4 @@ which gnome-autogen.sh || {
 
 REQUIRED_AUTOMAKE_VERSION=1.10
 REQUIRED_GTK_DOC_VERSION=1.13
-USE_GNOME2_MACROS=1
 . gnome-autogen.sh
