@@ -28,6 +28,11 @@
 #include "ev-view-marshal.h"
 #include "ev-document-misc.h"
 
+#if WITH_GSPELL
+#include <glib/gi18n.h>
+#include <gspell/gspell.h>
+#endif
+
 enum {
 	PROP_0,
 	PROP_ANNOTATION,
@@ -60,6 +65,11 @@ struct _EvAnnotationWindow {
 	gint          y;
 	gint          orig_x;
 	gint          orig_y;
+
+#if WITH_GSPELL
+	GspellTextView *spellcheck_view;
+	gboolean      enable_spellchecking;
+#endif
 };
 
 struct _EvAnnotationWindowClass {
@@ -354,6 +364,13 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 	/* Contents */
 	swindow = gtk_scrolled_window_new (NULL, NULL);
 	window->text_view = gtk_text_view_new ();
+
+#if WITH_GSPELL
+	window->spellcheck_view = NULL;
+	window->spellcheck_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (window->text_view));
+	gspell_text_view_basic_setup (window->spellcheck_view);
+#endif
+
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (window->text_view), GTK_WRAP_WORD);
 	g_signal_connect (window->text_view, "state-flags-changed",
 			  G_CALLBACK (text_view_state_flags_changed),
@@ -488,6 +505,9 @@ ev_annotation_window_constructor (GType                  type,
 			  G_CALLBACK (ev_annotation_window_opacity_changed),
 			  window);
 
+#if WITH_GSPELL
+        gspell_text_view_set_inline_spell_checking (window->spellcheck_view, ev_annotation_window_get_enable_spellchecking (window));
+#endif
 	return object;
 }
 
@@ -566,6 +586,9 @@ ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 	gtk_widget_class->focus_out_event = ev_annotation_window_focus_out_event;
         gtk_widget_class->key_press_event = ev_annotation_window_key_press_event;
 
+#if GTK_CHECK_VERSION(3, 20, 0)
+	gtk_widget_class_set_css_name (gtk_widget_class, "evannotationwindow");
+#endif
 	g_object_class_install_property (g_object_class,
 					 PROP_ANNOTATION,
 					 g_param_spec_object ("annotation",
@@ -694,3 +717,29 @@ ev_annotation_window_ungrab_focus (EvAnnotationWindow *window)
 
 	ev_annotation_window_sync_contents (window);
 }
+
+void
+ev_annotation_window_set_enable_spellchecking (EvAnnotationWindow *window,
+                                               gboolean enable_spellchecking)
+{
+        g_return_if_fail (EV_IS_ANNOTATION_WINDOW (window));
+
+#if WITH_GSPELL
+        if (enable_spellchecking == ev_annotation_window_get_enable_spellchecking (window))
+                return;
+
+        window->enable_spellchecking = enable_spellchecking;
+        gspell_text_view_set_inline_spell_checking (window->spellcheck_view, enable_spellchecking);
+#endif
+}
+
+gboolean
+ev_annotation_window_get_enable_spellchecking (EvAnnotationWindow *window)
+{
+        g_return_val_if_fail (EV_IS_ANNOTATION_WINDOW (window), FALSE);
+#if WITH_GSPELL
+        return window->enable_spellchecking;
+#else
+        return FALSE;
+#endif
+}
