@@ -48,8 +48,8 @@ G_BEGIN_DECLS
 #define EV_STOCK_INVERTED_COLORS        "inverted"
 #define EV_STOCK_ATTACHMENT             "mail-attachment"
 #define EV_STOCK_SEND_TO                "document-send"
-#define EV_STOCK_VIEW_SIDEBAR           "view-sidebar"
-#define EV_STOCK_OUTLINE                "outline"
+#define EV_STOCK_VIEW_SIDEBAR           "view-sidebar-symbolic"
+#define EV_STOCK_OUTLINE                "outline-symbolic"
 
 void ev_stock_icons_init       (void);
 void ev_stock_icons_shutdown   (void);
