@@ -594,8 +594,9 @@ ev_transition_animation_paint (EvTransitionAnimation *animation,
 		enum_value = g_enum_get_value (g_type_class_peek (EV_TYPE_TRANSITION_EFFECT_TYPE), type);
 
 		g_warning ("Unimplemented transition animation: '%s', "
-			   "please post a bug report in Evince bugzilla "
-			   "(http://bugzilla.gnome.org) with a testcase.",
+			   "please post a bug report in the issue tracker "
+			   "(https://gitlab.gnome.org/GNOME/evince/issues/) "
+			   "with a testcase.",
 			   enum_value->value_nick);
 
 		/* just paint the destination slide */
