@@ -127,6 +127,9 @@ void           ev_view_cancel_add_annotation (EvView          *view);
 void           ev_view_remove_annotation     (EvView          *view,
 					      EvAnnotation    *annot);
 gboolean       ev_view_add_text_markup_annotation_for_selected_text (EvView  *view);
+void           ev_view_set_enable_spellchecking (EvView *view,
+                                                 gboolean spellcheck);
+gboolean       ev_view_get_enable_spellchecking (EvView *view);
 
 /* Caret navigation */
 gboolean       ev_view_supports_caret_navigation    (EvView  *view);
